require 'spec_helper'
require 'yaml'

describe SoarAuthenticationToken::StaticTokenValidator do
  subject { SoarAuthenticationToken::StaticTokenValidator }
  before :all do
    @token_for_client_service_1         = 'some_secret_token_string_1111'
    @token_for_client_service_2         = 'some_secret_token_string_2222'
    @token_for_client_service_3_expired = 'some_secret_token_string_3333_expired'
    @token_for_client_service_3_unknown = 'some_secret_token_string_3333_unknown'
  end

  it 'has a version number' do
    expect(SoarAuthenticationToken::VERSION).not_to be nil
  end

  describe "#validate" do
    context "given that the validator is configured for static tokens" do
      let(:configuration) {
        current_time = Time.now
        {
          'provider' => 'SoarAuthenticationToken::StaticTokenValidator',
          'static_tokens' => [
            {
              'token'                    => 'some_secret_token_string_1111',
              'authenticated_identifier' => 'calling_client_service_1',
              'token_issue_time'         =>  current_time.utc.iso8601(3),
              'token_expiry_time'        => (current_time + 100).utc.iso8601(3)
            },
            {
              'token'                    => 'some_secret_token_string_2222',
              'authenticated_identifier' => 'calling_client_service_2',
              'token_issue_time'         =>  current_time.utc.iso8601(3),
              'token_expiry_time'        => (current_time + 100).utc.iso8601(3)
            },
            {
              'token'                    => 'some_secret_token_string_3333_expired',
              'authenticated_identifier' => 'calling_client_service_3',
              'token_issue_time'         => (current_time - 100).utc.iso8601(3),
              'token_expiry_time'        => (current_time - 50).utc.iso8601(3)
            }
          ]
        }
      }
      let(:iut) { subject.new(configuration) }
      context "given a token that is in the list of static tokens and not expired" do
        let(:response)       { iut.validate(authentication_token: @token_for_client_service_1) }
        let(:token_validity) { response[0] }
        let(:token_meta)     { response[1] }
        let(:message)        { response[2] }

        it 'should respond with true indicating token is valid' do
          expect(token_validity).to eq true
        end

        it 'should respond with token meta' do
          expect(token_meta).to_not eq nil
        end

        it 'should respond with a message indicating that the token is valid' do
          expect(message).to eq 'Valid token for <calling_client_service_1>'
        end
      end

      context "the token is in the list of static tokens and expired" do
        let(:response)       { iut.validate(authentication_token: @token_for_client_service_3_expired) }
        let(:token_validity) { response[0] }
        let(:token_meta)     { response[1] }
        let(:message)        { response[2] }

        it 'should respond with false indicating token is invalid' do
          expect(token_validity).to eq false
        end

        it 'should respond with nil token meta' do
          expect(token_meta).to eq nil
        end

        it 'should respond with a message indicating that the token is expired' do
          expect(message).to match /Expired token/
        end
      end

      context "given a token that is not in the list of static tokens" do
        let(:response)       { iut.validate(authentication_token: @token_for_client_service_3_unknown) }
        let(:token_validity) { response[0] }
        let(:token_meta)     { response[1] }
        let(:message)        { response[2] }

        it 'should respond with false indicating token is invalid' do
          expect(token_validity).to eq false
        end

        it 'should respond with no token meta' do
          expect(token_meta).to eq nil
        end

        it 'should respond with a message indicating that the token is valid' do
          expect(message).to match /Unknown static token/
        end
      end
    end
  end
end
